import watch from 'glob-watcher';
import { globbyStream } from 'globby';
import { NormalizedEntry } from './NormalizedEntry.js';
export class EntriesResolver {
    context;
    entriesFactory;
    constructor(context, entriesFactory) {
        this.context = context;
        this.entriesFactory = entriesFactory;
    }
    async update(isWatch) {
        const watchers = [];
        let invalidate;
        let close;
        const invalidatePromise = new Promise((resolve, reject) => {
            invalidate = () => {
                if (invalidate === undefined) {
                    return;
                }
                invalidate = undefined;
                for (const watcher of watchers) {
                    watcher.close();
                }
                resolve();
            };
            close = () => {
                if (invalidate === undefined) {
                    return;
                }
                invalidate = undefined;
                for (const watcher of watchers) {
                    watcher.close();
                }
                reject(new Error('Closed'));
            };
        });
        const factoryResult = await this.entriesFactory({
            context: this.context,
            glob: (patterns, options = {}) => {
                const cwd = options.cwd ?? this.context;
                const stream = async function* () {
                    if (isWatch) {
                        watchers.push(watch(patterns, {
                            cwd,
                            ignoreInitial: true,
                            events: ['add', 'unlink'],
                        }, () => {
                            invalidate?.();
                        }));
                    }
                    for await (const item of globbyStream(patterns, {
                        cwd,
                    })) {
                        yield item.toString('utf-8');
                    }
                }();
                let promise;
                const getPromise = () => {
                    if (promise === undefined) {
                        promise = (async () => {
                            const acc = [];
                            for await (const item of stream) {
                                acc.push(item);
                            }
                            return acc;
                        })();
                    }
                    return promise;
                };
                return {
                    then: (onResolve, onReject) => getPromise().then(onResolve, onReject),
                    catch: (onReject) => getPromise().catch(onReject),
                    [Symbol.asyncIterator]: stream[Symbol.asyncIterator].bind(stream),
                };
            }
        });
        let entriesMap = {};
        if (typeof factoryResult[Symbol.iterator] === 'function') {
            for (const { name, ...entry } of factoryResult) {
                entriesMap[name] = entry;
            }
        }
        else if (typeof factoryResult[Symbol.asyncIterator] === 'function') {
            for await (const { name, ...entry } of factoryResult) {
                entriesMap[name] = entry;
            }
        }
        else {
            for (const [name, entryMeta] of Object.entries(factoryResult)) {
                if (typeof entryMeta === 'string') {
                    entriesMap[name] = { import: [entryMeta] };
                }
                else {
                    entriesMap[name] = entryMeta;
                }
            }
        }
        const entries = Object.entries(entriesMap).map(([name, entry]) => new NormalizedEntry(entry.context ?? this.context, name, {
            ...entry,
            dependOn: typeof entry.dependOn === 'string' ? [entry.dependOn] : entry.dependOn,
            import: typeof entry.import === 'string' ? [entry.import] : entry.import,
        }));
        return {
            entries,
            invalidatePromise,
            close,
        };
    }
}
//# sourceMappingURL=EntriesResolver.js.map